/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 */

#ifndef COLOR_HPP
#define COLOR_HPP

#include <string>

namespace cpe
{

/** \brief octet is stored internaly as an unsigned char (1 byte) */
typedef unsigned char octet;

/** Storage class for a color
 *  Store the color as (r,g,b) value.
 *  Each component belongs in [0,255]
 */
class color
{
public:

    // ********************************************* //
    // ********************************************* //
    //  CONSTRUCTORS
    // ********************************************* //
    // ********************************************* //

    /** \brief Generate a color, value=(0,0,0)=black */
    color();
    /** \brief Direct constructor with values */
    color(octet r,octet g,octet b);
    /** \brief Direct constructor with values as integer (truncated in [0,255]) */
    color(int r,int g,int b);



    // ********************************************* //
    // ********************************************* //
    //  GET/SET
    // ********************************************* //
    // ********************************************* //

    /** \brief get the red value */
    int red() const;
    /** \brief get the green value */
    int green() const;
    /** \brief get the blue value */
    int blue() const;

    /** \brief set the red value */
    octet& red();
    /** \brief set the green value */
    octet& green();
    /** \brief set the blue value */
    octet& blue();

    // ********************************************* //
    // ********************************************* //
    //  COMPUTATION
    // ********************************************* //
    // ********************************************* //

    /** \brief Compute the norm between two colors
     *
     *  Compute the Euclidian distance sqrt((r2-r1)^2+(g2-g1)^2+(b2-b1)^2)
     */
    static double norm(const color& c1,const color& c2);


    // ********************************************* //
    // ********************************************* //
    //  Conversion
    // ********************************************* //
    // ********************************************* //

    /** \brief convert the color into string */
    std::string to_string() const;

private:

    /** \brief internal storage for red component */
    octet r;
    /** \brief internal storage for green component */
    octet g;
    /** \brief internal storage for blue component */
    octet b;
};

/** \brief write the color value into the stream */
std::ostream& operator<<(std::ostream& stream,const color& c);
}

#endif
