#include <image.hpp>

#include <iostream>
#include <assert.h>

namespace cpe
{
image::image()
    :size_Nx(0),size_Ny(0)
{}

image::image(unsigned int Nx_,unsigned int Ny_)
    :size_Nx(Nx_),size_Ny(Ny_)
{
    resize(size_Nx,size_Ny);
}
image::image(unsigned int Nx_,unsigned int Ny_,const std::vector<color> data_)
    :size_Nx(Nx_),size_Ny(Ny_),data(data_)
{
    assert(data.size()==Nx_*Ny_);
}

void image::resize(unsigned int Nx_,unsigned int Ny_)
{
    data.resize(Nx_*Ny_);
}

void image::set_raw_data(const std::vector<octet>& raw_data)
{
    assert(raw_data.size()==size_Nx*size_Ny*3 || raw_data.size()==size_Nx*size_Ny*4);

    if(raw_data.size()==size_Nx*size_Ny*4)
        for(unsigned int k=0;k<size_Nx*size_Ny;++k)
            data[k]=color(raw_data[4*k+0],raw_data[4*k+1],raw_data[4*k+2]);
    else if(raw_data.size()==size_Nx*size_Ny*3)
        for(unsigned int k=0;k<size_Nx*size_Ny;++k)
            data[k]=color(raw_data[3*k+0],raw_data[3*k+1],raw_data[3*k+2]);
}

const color& image::get(const unsigned int kx,const unsigned ky) const
{
    assert(kx<size_Nx && ky<size_Ny);

    return data[kx+size_Nx*ky];
}


const color& image::operator()(const unsigned int kx,const unsigned int ky) const
{
    return get(kx,ky);
}
color& image::operator()(const unsigned int kx,const unsigned int ky)
{
    assert(kx<size_Nx && ky<size_Ny);
    return data[kx+size_Nx*ky];
}

std::vector<octet> image::to_raw_data_rgba() const
{
    std::vector<octet> raw(size_Nx*size_Ny*4);
    for(unsigned int k=0;k<size_Nx*size_Ny;++k)
    {
        raw[4*k+0]=data[k].red();
        raw[4*k+1]=data[k].green();
        raw[4*k+2]=data[k].blue();
        raw[4*k+3]=255;
    }
    return raw;
}
std::vector<octet> image::to_raw_data_rgb() const
{
    std::vector<octet> raw(size_Nx*size_Ny*3);
    for(unsigned int k=0;k<size_Nx*size_Ny;++k)
    {
        raw[3*k+0]=data[k].red();
        raw[3*k+1]=data[k].green();
        raw[3*k+2]=data[k].blue();
    }
    return raw;

}

unsigned int image::Nx() const
{
    return size_Nx;
}
unsigned int image::Ny() const
{
    return size_Ny;
}

void image::set(const unsigned int kx,const unsigned ky,const color& c)
{
    assert(kx<size_Nx && ky<size_Ny);
    data[kx+size_Nx*ky]=c;
}

void image::fill(const color& c)
{
    assert(data.size()==size_Nx*size_Ny);

    for(unsigned int k=0;k<data.size();++k)
        data[k]=c;
}

void image::fill_bloc(unsigned int kx,unsigned int ky,unsigned int size_bloc,const color& c)
{
    assert(data.size()==size_Nx*size_Ny);
    assert(kx<size_Nx && ky<size_Ny);
    assert(kx+size_bloc-1<size_Nx && ky+size_bloc-1<size_Ny);

    for(unsigned int dx=0;dx<size_bloc;++dx)
        for(unsigned int dy=0;dy<size_bloc;++dy)
            set(kx+dx,ky+dy,c);
}

void image::copy_bloc(unsigned int kx_source,unsigned int ky_source,const image& image_source,
                      unsigned int kx_destination,unsigned int ky_destination,image* p_image_destination,
                      unsigned int size_bloc)
{
    assert(p_image_destination!=NULL);
    image& image_destination=*p_image_destination;

    assert(kx_source<image_source.Nx() && ky_source<image_source.Ny());
    assert(kx_destination<image_destination.Nx() && ky_destination<image_destination.Ny());

    assert(kx_source+size_bloc-1<image_source.Nx() && ky_source+size_bloc-1<image_source.Ny());
    assert(kx_destination+size_bloc-1<image_destination.Nx() && ky_destination+size_bloc-1<image_destination.Ny());

    for(unsigned int dx=0;dx<size_bloc;++dx)
    {
        for(unsigned int dy=0;dy<size_bloc;++dy)
        {
            image_destination.set(kx_destination+dx,ky_destination+dy,image_source(kx_source+dx,ky_source+dy));
        }
    }

}



}
